{% extends "admin/change_form.html" %}
{% load i18n admin_modify admin_urls %}{% load url from aa_compat %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
  <a href="../../">{% trans "Home" %}</a> &rsaquo;
  <a href="../">{{ app_label|capfirst|escape }}</a> &rsaquo;
  <a href=".">{{ opts.verbose_name_plural|capfirst }}</a> &rsaquo;
  {{ action_short_description|capfirst }}
</div>
{% endif %}{% endblock %}

{% block content %}
{% if form.subject.errors %}
<ol>
  {% for error in form.subject.errors %}
  <li><strong>{{ error|escape }}</strong></li>
  {% endfor %}
</ol>
{% endif %}
{{ data }}
<div id='form'>
  <form method="post">
    {% csrf_token %}
    <table>
      {{ adminform.form }}
    </table>
    <input style="margin: 5px;" type="submit" name="apply" value="{% trans 'Export' %}"/>
  </form>
</div>
{% endblock %}
