{% extends "admin/change_form.html" %}
{% load i18n admin_modify admin_urls %}{% load url from aa_compat %}
{% block extrahead %}{{ block.super }}
<style type="text/css">
  #form {
    float: left;
    width: 40%;
  }

  #legend {
    float: left;
    width: 60%;
  }
</style>
{% endblock %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
  <a href="{% url 'admin:index' %}">{% trans "Home" %}</a> &rsaquo;
  <a href="{% url 'admin:index' %}{{ app_label}}">{{ app_label|capfirst }}</a> &rsaquo;
  {% if has_change_permission %}<a
  href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst }}</a>{% else %}{{
  opts.verbose_name_plural|capfirst }}{% endif %} &rsaquo;
  {{ action_short_description|capfirst }}
</div>
{% endif %}{% endblock %}

{% block content %}
{% if adminform.form.subject.errors %}
<ol>
  {% for error in adminform.form.subject.errors %}
  <li><strong>{{ error|escape }}</strong></li>
  {% endfor %}
</ol>
{% endif %}
<div id='form'>
  <form action="" method="post">
    {% csrf_token %}
    <table>
      {{ adminform.form }}
    </table>
    <input style="margin: 5px;" type="submit" name="apply" value="{% trans 'Export' %}"/>
  </form>
</div>
{% endblock %}
