.mergetable {
    width: 100%;
    border: 1px solid black;
}
p.display {
    width: 300px;
    overflow: hidden;
    white-space: nowrap;
    /*word-wrap: break-word;*/
}

.mergetable tr.header th {
    font-weight: bold;
    text-align: center;
}

.mergetable td {
    /*width: 30%;*/
    vertical-align: middle;
    border-right: 1px solid #000000;
}

.mergetable .column {
     /*width: 30%;*/
}

.mergetable .origin *, .mergetable .result * {
    /*width: 30%;*/
    /*word-wrap: break-word;*/
    /*white-space:pre-line;*/
}

.mergetable .label {
    /*width: 10%;*/
    font-weight: bold;
}

.mergetable .selected {
    background-color: #81b10c;
}

.mergetable .changed {
    background-color: #f9adaf;
}

.mergetable input.raw-value {
    /*width: 200px;*/
    border: 0px transparent;
    background-color: transparent;
    display: none;
}
